package com.iflytek.cloud;

import android.content.Context;
import android.graphics.Bitmap;

import com.iflytek.cloud.msc.module.SpeechInterface;
import com.iflytek.cloud.msc.util.log.DebugLog;
import com.iflytek.cloud.speech.impl.FaceDetectorImpl;

/**
 * <h3>本地人脸检测类</h3>
 * <script>gShowExtraFace();</script>
 * <p>
 * 		包含图片检测、视频流检测功能。本地人脸图片检测有两种方式一种是灰度图检测，一种是ARGB图检测。
 * 灰度图检测方法传入的必须为灰度图，通过{@link com.iflytek.cloud.util.VerifierUtil}类将ARGB图转换
 * 成灰度图。检测返回的结果说明，请参考<script>gShowUserGuideFile();</script>。
 * </p>
 * <p>
 * 		当前本地人脸图片检测支持检测的Bitmap格式数据，故支持以下图片格式：PNG，BMP, JPG。
 * 不需要针对不同格式设置格式参数，只需将图片数据保存为Bitmap格式即可。且目前本地人脸图片
 * 检测最大支持1500万像素的图片，若图片像素高于1500王请开发者参考FaceDemo对图片进行压缩。
 * 本地视频流检测仅支持检测nv21格式数据，且支持分辨率大小为640*480。
 * </p>
 * <p>
 * 		本类使用单例，调用者使用本类的对象，只需要通过{@link #createDetector}创建
 * 一次对象后，便可一直使用该对象，直到通过调用{@link #destroy}进行单例对象销毁。调
 * 用者可通过{@link #getDetector}获取当前已经创建的单例。在销毁本类的单例对象后，
 * 需要先通过{@link #createDetector}再次创建单例对象，方可再使用。
 * </p>
 * <p>
 * 		在当前应用生命周期第一次使用本类的任何函数前，须先调用
 * {@link SpeechUtility#createUtility}进行SDK初始化。
 * </p>
 * 
 * @author <script>gShowAuthor();</script>
 * @since version:1073
 * @see SpeechUtility#createUtility
 */
public class FaceDetector extends SpeechInterface {
	private static FaceDetector sInstance;
	private FaceDetectorImpl mFaceDetectorImpl;
	
	private FaceDetector(Context context) throws SpeechError {
		try{
			mFaceDetectorImpl = new FaceDetectorImpl(context, null);
		}catch (UnsatisfiedLinkError err) {
			throw new SpeechError(ErrorCode.ERROR_UNSATISFIED_LINK);
		}

	}
	
	/**
	 * <h4>创建单例对象</h4>
	 * <p>
	 * 		使用此函数创建一个本类单例对象。当成功创建一次单例对象后，可一直使用此对象，
	 * 直到调用{@link #destroy}销毁已创建的单例对象为止。若在当前应用生命周期内调用
	 * {@link #destroy}前再次调用本函数，则直接返回已创建的单例对象。可通过
	 * {@link #getDetector()}获取已创建的单例对象。
	 * </p>
	 * 
	 * @param context
	 * @param params
	 * @return FaceDetector 单例对象
	 * 
	 * @see #destroy
	 * @see #getDetector
	 */
	public static synchronized FaceDetector createDetector(Context context, String params) throws SpeechError{
		FaceDetector instance = null;
		
		synchronized( sSync ){
			if( null==sInstance && null!=SpeechUtility.getUtility() ) {
				sInstance = new FaceDetector(context);
			}
			
			instance = sInstance;
		}//end of sync
		
		return instance;
	}
	
	/**
	 * <h4>获取单例对象</h4>
	 * <p>
	 * 		通过函数获取已创建的单例对象。当单例对象未创建时，将返回null，此时应先通过
	 * {@link #createDetector}创建单例对象。
	 * </p>
	 * 
	 * @return 人脸检测对象
	 * 
	 * @see #createDetector
	 * @see #destroy
	 */
	public static synchronized FaceDetector getDetector(){
		return sInstance;
	}
	
	/**
	 * <h4>检测ARGB图片</h4>
	 * <p>
	 * 		检测ARGB图片中的人脸。
	 * </p>
	 * 
	 * @param src ARGB人脸图片
	 * @return 人脸检测结果
	 * 
	 * @see #getDetector()
	 * @see #detectGray(Bitmap)
	 */
	public synchronized String detectARGB(Bitmap src) {
		String ret = null;
		
		synchronized( this ){
			if( null != this.mFaceDetectorImpl ){
				ret = mFaceDetectorImpl.detectARGB(src);
			}
		}//end of sync
		
		return ret;
	}
	
	/**
	 * <h4>检测灰度图片</h4>
	 * <p>
	 * 		检测灰度图片中的人脸。
	 * </p>
	 * 
	 * @param src 灰度人脸图片
	 * @return 人脸检测结果
	 * 
	 * @see #getDetector()
	 * @see #detectARGB(Bitmap)
	 */
	public synchronized String detectGray(Bitmap src) {
		String ret = null;
		
		synchronized( this ){
			if( null != this.mFaceDetectorImpl ){
				ret = mFaceDetectorImpl.detectGray(src);
			}
		}//end of sync
		
		return ret;
	}
	
	/**
	 * <h4>检测nv21视频帧中的人脸</h4>
	 * <p>
	 * 		对摄像头视频流中的人脸进行实时检测。本接口可以通过设置参数进行聚焦控制，
	 * 即在结果中是否包含关键点信息。在进行检测之前，需要创建并开启重力感应器对象
	 * {@link com.iflytek.cloud.util.Accelerometer}，用于获取设备的朝向。
	 * SDK暂时只支持640*480的nv21视频帧，传入其他格式（尺寸）的视频都不能正常工作。
	 * </p>
	 * 
	 * @param nv21 视频帧数据
	 * @param width 视频帧图像宽
	 * @param height 视频帧图像高
	 * @param align 聚焦控制
	 * @param dir 图像的方向
	 * @return 人脸检测结果
	 * 
	 * @see #detectARGB(Bitmap)
	 */
	public synchronized String trackNV21(byte[] nv21, int width, int height, int align,int dir) {
		String ret = null;
		
		synchronized( this ){
			if( null != this.mFaceDetectorImpl ){
				ret = mFaceDetectorImpl.trackNV21(nv21, width, height, align,dir);
			}
		}//end of sync
		
		return ret; 
	}
	
	/**
	 * <h4>销毁单例对象</h4>
	 * <p>
	 * 		通过本函数，销毁由{@link #createDetector}创建的单例对象。
	 * 此时需要再使用，应先通过{@link #createDetector}创建一个新的单例
	 * 对象。
	 * </p>
	 * 
	 * @see #createDetector
	 */
	public synchronized boolean destroy() {
		final FaceDetectorImpl impl = mFaceDetectorImpl;
		
		synchronized( this ){
			mFaceDetectorImpl = null;
		}//end of sync
		
		if( null != impl ){
			impl.destroy();
		}
		
		boolean ret = super.destroy();
		if( ret ){
			synchronized( sSync ){	
				sInstance = null;
			}//end of sync
		}
		
		return ret;
	}
}
